#!/bin/bash
# MODIFICATIONS
# 29-09-09
# - Better script restarting behaviour
# - fixed dependancies with dahdi hardware (Florian)
# - Added explanation for multiple MYSQL password requests (Michal)
# - Various typos and explanations changed
#
# 03-03-2009
# - can restart script and archives will either resume or skip based on 
#   whether they are completed or not.
# - removed some prompts that slowed the install waiting for user input
# - added ssh server install
# - added beep command for audio feedback during install
# - removed sql change password script (moved to a seperate script)
# - fixed issue with max_upload in php.ini not updating
# - added magic_quotes_gpc change required for FPBX 2.5
# - removed dependancy on specific kernel version 
# - (script will check what you are running and sort out the rest)
# - added ability to input mysql password when creating amportal tables

clear
# you need to update these version numbers from the Asterisk download site to make sure
# the current version is being installed (www.asterisk.com/downloads)
VER_ASTERISK="1.4.33.1";
VER_DAHDI_LINUX="2.3.0.1";
VER_DAHDI_TOOLS="2.3.0";
VER_LIBPRI="1.4.11";
VER_ADDONS="1.6.2.1"; 
VER_FREEPBX="2.7.0";
# no more editing below this line unless you are brave :)
KERNEL_VERSION=`uname -r`
KERNEL_VERSION2=`uname -r`
KERNEL_VERSION=`echo ${KERNEL_VERSION}  | sed -e "s/\-\(.*\)//"`
DEBIAN_VERSION=`cat /etc/debian_version`


echo "##################################################################"
echo "#    MODIFIED Automated installer for Asterisk with FreePBX      #"
echo "#           This is the DAHDI VERSION OF THE SCRIPT              #"
echo "#          BY:   Matt Chipman matt@corenetworks.com.au           #"
echo "#         originally based on the excellent work done by         #"
echo "#                                                                #"
echo "#                Yoann QUERET - yoann@queret.net                 #"
echo "#                http://www.queret.net/asterisk/                 #"
echo "#                                                                #"
echo "#             == Contributors to the original script ==          #"
echo "#               Serge Berney - s.berney@kinonline.net            #"
echo "#              Neil Stone - nstone@freewayprojects.com           #"
echo "#                                                                #"
echo "#         mods can be viewed by reading this source file         #"
echo "### ---------------------------------------------------------- ###"
echo "-> Asterisk.......${VER_ASTERISK}"
echo "-> Dahdi-Linux....${VER_DAHDI_LINUX}"
echo "-> Dahdi-Tools....${VER_DAHDI_TOOLS}"
echo "-> Libpri.........${VER_LIBPRI}"
echo "-> Addons.........${VER_ADDONS}"
echo "-> FreePBX........${VER_FREEPBX} (+ apache2 php5 mysql5)"
echo "### ---------------------------------------------------------- ###"
echo "#                    Press <enter> to continue                   #"
echo "##################################################################"
read
clear

echo "##################################################################"
echo "#                  == SYSTEM CONFIGURATION ==                    #"
echo "#     Debian ${DEBIAN_VERSION} with kernel ${KERNEL_VERSION}            #"
echo "#                                                                #"
echo "#                  == NETWORK CONFIGURATION ==                   #"
echo "#   Your server must have an active network connection           #"
echo "#                                                                #"
echo "#                     == Disclaimer ==                           #"
echo "#        This script built and tested on Debian 5.0 (Lenny)      #"
echo "#      Everyone should be using Lenny for new installs anyway    #"
echo "### ---------------------------------------------------------- ###"
echo "#                     Press <enter> to continue                  #"
echo "##################################################################"
read
clear

echo "##################################################################"
echo "#          CNTRL C to exit or we are installing now              #"
echo "#               THIS IS YOUR LAST CHANCE TO EXIT                 #"
echo "### ---------------------------------------------------------- ###"
echo "#    The script will run by itself now for about 20 minutes      #"
echo "#	If you have a decent Internet connection it will be quicker    #"
echo "#								       #"
echo "#                   Press <enter> to continue                    #"
echo "##################################################################"
read
clear
echo "##################################################################"
echo "# PART 1 - Asterisk installation -- Please wait a moment ....    #"
echo "##################################################################"

aptitude update
aptitude -y install beep ssh
beep -f 500 -n -f 1000 -n -f 1500
beep -f 500 -n -f 1000 -n -f 1500

echo "+----------------------------------------------------------------+"
echo "| Download sources to /tmp/asterisk/                             |"
echo "+----------------------------------------------------------------+"
echo " "
echo " "
if [ -d /tmp/asterisk ]; then
	echo "hmmm it looks like you have run this script previously, we will try and pickup where we left off"
	echo "I will just make sure the source trees are clean"
	echo " "
	cd /usr/src/dahdi-linux-${VER_DAHDI_LINUX}
	make distclean
	echo "ok dahdi"
	echo " "
	cd /usr/src/dahdi-tools-${VER_DAHDI_TOOLS}
	make distclean
	echo "ok dahdi-tools"
	echo" "
	cd /usr/src/libpri-${VER_LIBPRI}
	make clean
	echo "ok libpri"
	echo " "
	cd /usr/src/asterisk-${VER_ASTERISK}
	make distclean
	echo "ok asterisk"
	echo " "

else 
	mkdir /tmp/asterisk/
fi

wget -c http://downloads.asterisk.org/pub/telephony/asterisk/releases//asterisk-${VER_ASTERISK}.tar.gz -O /tmp/asterisk/asterisk-${VER_ASTERISK}.tar.gz
wget -c http://downloads.asterisk.org/pub/telephony/dahdi-linux/releases/dahdi-linux-${VER_DAHDI_LINUX}.tar.gz -O /tmp/asterisk/dahdi-linux-${VER_DAHDI_LINUX}.tar.gz
wget -c http://downloads.asterisk.org/pub/telephony/dahdi-tools/releases/dahdi-tools-${VER_DAHDI_TOOLS}.tar.gz -O /tmp/asterisk/dahdi-tools-${VER_DAHDI_TOOLS}.tar.gz
wget -c http://downloads.asterisk.org/pub/telephony/libpri/releases/libpri-${VER_LIBPRI}.tar.gz -O /tmp/asterisk/libpri-${VER_LIBPRI}.tar.gz
wget -c http://downloads.asterisk.org/pub/telephony/asterisk/releases/asterisk-addons-${VER_ADDONS}.tar.gz -O /tmp/asterisk/asterisk-addons-${VER_ADDONS}.tar.gz


echo "+----------------------------------------------------------------+"
echo "| Installing required packages                                   |"
echo "+----------------------------------------------------------------+"
aptitude -y install linux-source-${KERNEL_VERSION} kernel-package g++ libncurses5-dev linux-headers-${KERNEL_VERSION2} sqlite libnewt-dev libusb-dev zlib1g-dev libmysqlclient15-dev libsqlite0-dev

echo "+----------------------------------------------------------------+"
echo "| Configuring of kernel sources                                  |"
echo "+----------------------------------------------------------------+"
cd /usr/src/
tar xvjf linux-source-${KERNEL_VERSION}.tar.bz2
ln -s /usr/src/linux-source-${KERNEL_VERSION} /usr/src/linux
cd /usr/src/linux/
make oldconfig
make prepare
make prepare scripts

echo "+----------------------------------------------------------------+"
echo "| Decompresing sources in /usr/src/                              |"
echo "+----------------------------------------------------------------+"
cd /usr/src/
tar zxvf /tmp/asterisk/asterisk-${VER_ASTERISK}.tar.gz
tar zxvf /tmp/asterisk/asterisk-addons-${VER_ADDONS}.tar.gz
tar zxvf /tmp/asterisk/libpri-${VER_LIBPRI}.tar.gz 
tar zxvf /tmp/asterisk/dahdi-linux-${VER_DAHDI_LINUX}.tar.gz
tar zxvf /tmp/asterisk/dahdi-tools-${VER_DAHDI_TOOLS}.tar.gz

echo "+----------------------------------------------------------------+"
echo "| Compiling Dahdi-linux                                          |"
echo "+----------------------------------------------------------------+"
cd /usr/src/dahdi-linux-${VER_DAHDI_LINUX}

make
make install


echo "+----------------------------------------------------------------+"
echo "| Compiling Dahdi-tools                                          |"
echo "+----------------------------------------------------------------+"
cd /usr/src/dahdi-tools-${VER_DAHDI_TOOLS}
./configure

make
make install
make config


echo "+----------------------------------------------------------------+"
echo "| Compiling libpri                                               |"
echo "+----------------------------------------------------------------+"
cd /usr/src/libpri-${VER_LIBPRI}
make install

echo "+----------------------------------------------------------------+"
echo "| Compiling Asterisk                                             |"
echo "+----------------------------------------------------------------+"
cd /usr/src/asterisk-${VER_ASTERISK}
./configure

sed -i "s/PROC=armv5tel/PROC=arm/" /usr/src/asterisk-${VER_ASTERISK}/makeopts


MAKEOPTS="/usr/src/asterisk-${VER_ASTERISK}/menuselect.makeopts"

echo 'MENUSELECT_APPS=app_ivrdemo app_osplookup app_rpt app_skel' >> ${MAKEOPTS}
echo 'MENUSELECT_CDR=cdr_odbc cdr_pgsql cdr_radius cdr_tds' >> ${MAKEOPTS}
echo 'MENUSELECT_CHANNELS=chan_alsa chan_gtalk chan_h323 chan_misdn chan_nbs chan_vpb' >> ${MAKEOPTS}
echo 'MENUSELECT_CODECS=codec_ilbc codec_speex' >> ${MAKEOPTS}
echo 'MENUSELECT_FORMATS=format_ogg_vorbis' >> ${MAKEOPTS}
echo 'MENUSELECT_FUNCS=func_curl func_odbc' >> ${MAKEOPTS}
echo 'MENUSELECT_PBX=pbx_gtkconsole' >> ${MAKEOPTS}
echo 'MENUSELECT_RES=res_config_odbc res_config_pgsql res_crypto res_jabber res_odbc res_snmp' >> ${MAKEOPTS}
echo 'MENUSELECT_OPTS_app_voicemail=' >> ${MAKEOPTS}
echo 'MENUSELECT_CFLAGS=LOADABLE_MODULES' >> ${MAKEOPTS}
echo 'MENUSELECT_EMBED=apps cdr channels codecs formats funcs pbx res' >> ${MAKEOPTS}
echo 'MENUSELECT_CORE_SOUNDS=CORE-SOUNDS-EN-WAV CORE-SOUNDS-EN-ULAW CORE-SOUNDS-EN-ALAW CORE-SOUNDS-EN-GSM CORE-SOUNDS-EN-G729 CORE-SOUNDS-EN-G722' >> ${MAKEOPTS}
echo 'MENUSELECT_MOH=MOH-FREEPLAY-WAV MOH-FREEPLAY-ULAW MOH-FREEPLAY-ALAW MOH-FREEPLAY-GSM MOH-FREEPLAY-G729 MOH-FREEPLAY-G722' >> ${MAKEOPTS}
echo 'MENUSELECT_EXTRA_SOUNDS=EXTRA-SOUNDS-EN-WAV EXTRA-SOUNDS-EN-ULAW EXTRA-SOUNDS-EN-ALAW EXTRA-SOUNDS-EN-GSM EXTRA-SOUNDS-EN-G729 EXTRA-SOUNDS-EN-G722' >> ${MAKEOPTS}
echo 'MENUSELECT_BUILD_DEPS=res_adsi chan_local res_indications app_meetme res_monitor res_smdi res_features' >> ${MAKEOPTS}

make install
make samples

echo "+----------------------------------------------------------------+"
echo "| Compiling Asterisk Addons                                      |"
echo "+----------------------------------------------------------------+"
cd /usr/src/asterisk-addons-${VER_ADDONS}
./configure


make install


echo "+----------------------------------------------------------------+"
echo "| Creating symlinks (Modules)                                    |"
echo "+----------------------------------------------------------------+"
ln -s /lib/modules/${KERNEL_VERSION}/ /lib/modules/`uname -r`/asterisk
depmod

echo "##################################################################"
echo "# PART1 : Asterisk installation : Done                           #"
echo "##################################################################"


###################################################################################################

###################################################################################################

###################################################################################################



clear
echo "##################################################################"
echo "# PART 2 - FreePBX installation                                  #"
echo "##################################################################"

echo "+----------------------------------------------------------------+"
echo "| Installing packages                                            |"
echo "+----------------------------------------------------------------+"

aptitude install -y apache2 php5 php5-cli mysql-server-5.0 php-pear php5-mysql php-db libapache2-mod-php5 php5-gd php5-curl

echo "+----------------------------------------------------------------+"
echo "| Downloading FreePBX source to /tmp/freepbx/                    |"
echo "+----------------------------------------------------------------+"

mkdir /tmp/freepbx/
wget -c http://mirror.freepbx.org/freepbx-${VER_FREEPBX}.tar.gz -O /tmp/freepbx/freepbx-${VER_FREEPBX}.tar.gz

echo "+----------------------------------------------------------------+"
echo "| Decompresing FreePBX sources in /usr/src/             |"
echo "+----------------------------------------------------------------+"

cd /usr/src
tar zxvf /tmp/freepbx/freepbx-${VER_FREEPBX}.tar.gz

echo "+----------------------------------------------------------------+"
echo "| Changing maximum upload limit in PHP                           |"
echo "+----------------------------------------------------------------+"

cp /etc/php5/apache2/php.ini /etc/php5/apache2/php.ini-orig
sed -i "s/upload_max_filesize = 20M/upload_max_filesize = 120M/" /etc/php5/apache2/php.ini
sed -i "s/magic_quotes_gpc = On/magic_quotes_gpc = Off/" /etc/php5/apache2/php.ini
sed -i "s/memory_limit = 16M/memory_limit = 100M/" /etc/php5/apache2/php.ini

echo "+----------------------------------------------------------------+"
echo "| Music on hold  (symlink)                                       |"
echo "+----------------------------------------------------------------+"

ln -s /var/lib/asterisk/moh /var/lib/asterisk/mohmp3

echo "+----------------------------------------------------------------+"
echo "| Setting permissions for asterisk user                          |"
echo "+----------------------------------------------------------------+"

adduser asterisk --disabled-password --gecos "asterisk PBX" --home /var/lib/asterisk
adduser www-data asterisk

echo "+----------------------------------------------------------------+"
echo "| Changing apache user and group to 'asterisk'                   |"
echo "+----------------------------------------------------------------+"
cp /etc/apache2/apache2.conf /etc/apache2/apache2.conf-orig
sed -i "s/\(^User *\)\(.*\)/\1asterisk/" /etc/apache2/apache2.conf
sed -i "s/\(^Group *\)\(.*\)/\1asterisk/" /etc/apache2/apache2.conf

clear
echo "+----------------------------------------------------------------+"
echo "| MySQL                                                          |"
echo "+----------------------------------------------------------------+"

# add mysql prompts 
echo "Create call records database (MYSQL root password please, 1 of 4)"
echo "create database asteriskcdrdb;" | mysql -u root -p
echo "Create Asterisk database (MYSQL root password please, 2 of 4)"
echo "create database asterisk;" | mysql -u root -p
echo "Create call datbase user (MYSQL root password please, 3 of 4)"
echo "GRANT ALL PRIVILEGES ON asteriskcdrdb.* TO asteriskuser@localhost IDENTIFIED BY 'amp109';" | mysql -u root -p
echo "Create Asterisk database user (MYSQL root password please, 4 of 4)"
echo "GRANT ALL PRIVILEGES ON asterisk.* TO asteriskuser@localhost IDENTIFIED BY 'amp109';" | mysql -u root -p

mysql -u asteriskuser -pamp109 asteriskcdrdb < /usr/src/freepbx-${VER_FREEPBX}/SQL/cdr_mysql_table.sql
mysql -u asteriskuser -pamp109 asterisk < /usr/src/freepbx-${VER_FREEPBX}/SQL/newinstall.sql

echo "+----------------------------------------------------------------+"
echo "| Configuring Asterisk to use FreePBX                            |"
echo "+----------------------------------------------------------------+"

cp /etc/asterisk/asterisk.conf /etc/asterisk/asterisk.conf.orig
sed -i "s/\(astrundir *=> *\)\(.*\)/\1\/var\/run\/asterisk/" /etc/asterisk/asterisk.conf

mkdir /var/run/asterisk
chown -R asterisk:asterisk /var/run/asterisk

echo "+----------------------------------------------------------------+"
echo "| Loading Asterisk Post Installation                             |"
echo "+----------------------------------------------------------------+"
/usr/sbin/asterisk

echo "+----------------------------------------------------------------+"
echo "| Modifying FreeBBX configuration files                          |"
echo "+----------------------------------------------------------------+"

cp /usr/src/freepbx-${VER_FREEPBX}/install_amp /usr/src/freepbx-${VER_FREEPBX}/install_amp-orig
sed -i "s/\(^\$webroot*\)\(.*\)/\1 = \"\/var\/www\";/" /usr/src/freepbx-${VER_FREEPBX}/install_amp
chmod 755 /usr/src/freepbx-${VER_FREEPBX}/install_amp

LOCAL_IP=`/sbin/ifconfig eth0`
LOCAL_IP=`echo ${LOCAL_IP} | sed -e "s/\ Bcast:\(.*\)//"`
LOCAL_IP=`echo ${LOCAL_IP} | sed -e "s/\(.*\)\ inet addr://"`
sed -i "s/xx.xx.xx.xx/${LOCAL_IP}/g" "/usr/src/freepbx-${VER_FREEPBX}/install_amp"

chmod 755 /usr/src/freepbx-${VER_FREEPBX}/install_amp

echo "+----------------------------------------------------------------+"
echo "| Installation of FreePBX                                        |"
echo "+----------------------------------------------------------------+"
echo "| Simply press <Enter> at each question.                         |"
echo "+----------------------------------------------------------------+"
echo " Press <enter> to continue"
read
cd /usr/src/freepbx-${VER_FREEPBX}/
./install_amp

/usr/src/freepbx-${VER_FREEPBX}/apply_conf.sh

echo "+----------------------------------------------------------------+"
echo "| Stop Asterisk Post Installation                                |"
echo "+----------------------------------------------------------------+"

asteriskPID=$(cat /var/run/asterisk/asterisk.pid)
kill -9 $(cat /var/run/asterisk/asterisk.pid)


echo "+----------------------------------------------------------------+"
echo "| Remove Apache2 Redirect                                        |"
echo "+----------------------------------------------------------------+"

sed -i "s/\(RedirectMatch*\)\(.*\)//" /etc/apache2/sites-enabled/000-default

echo "+----------------------------------------------------------------+"
echo "| Reload apache                                                  |"
echo "+----------------------------------------------------------------+"

/etc/init.d/apache2 restart

echo "+----------------------------------------------------------------+"
echo "| Changing permissions                                           |"
echo "+----------------------------------------------------------------+"
echo " Press <enter> to continue"

chown -R asterisk:asterisk /etc/asterisk
chmod 770 /etc/asterisk/

chown -R asterisk:asterisk /var/lib/asterisk/
chmod 770 /var/lib/asterisk/

chown -R asterisk:asterisk /var/www/


echo "+----------------------------------------------------------------+"
echo "| Copying missing images                                    |"
echo "+----------------------------------------------------------------+"
cp /var/www/admin/modules/dashboard/images/notify_* /var/www/admin/images/

echo "+----------------------------------------------------------------+"
echo "| Startup script (Asterisk+FrePBX)                                  |"
echo "| /etc/init.d/freepbx [start|stop|restart]                       |"
echo "+----------------------------------------------------------------+"
STARTUP_SCRIPT="/etc/init.d/freepbx";
echo "Creating file ${STARTUP_SCRIPT} ...";

echo '#!/bin/bash' > ${STARTUP_SCRIPT}
echo '' >> ${STARTUP_SCRIPT}
echo 'AMPORTAL_BIN=/usr/local/sbin/amportal' >> ${STARTUP_SCRIPT}
echo '' >> ${STARTUP_SCRIPT}
echo 'if [ ! -x ${AMPORTAL_BIN} ]; then'  >> ${STARTUP_SCRIPT}
echo '        echo "error : amportal binary can not be found (${AMPORTAL_BIN})"' >> ${STARTUP_SCRIPT}
echo '        exit 0' >> ${STARTUP_SCRIPT}
echo 'fi' >> ${STARTUP_SCRIPT}
echo '' >> ${STARTUP_SCRIPT}
echo '' >> ${STARTUP_SCRIPT}
echo 'start() {' >> ${STARTUP_SCRIPT}
echo '	echo "Starting FreePBX ..."' >> ${STARTUP_SCRIPT}
echo '	${AMPORTAL_BIN} start'  >> ${STARTUP_SCRIPT}
echo '}' >> ${STARTUP_SCRIPT}
echo '' >> ${STARTUP_SCRIPT}
echo 'stop() {' >> ${STARTUP_SCRIPT}
echo '	echo "Stopping FreePBX ..."' >> ${STARTUP_SCRIPT}
echo '	${AMPORTAL_BIN} stop'  >> ${STARTUP_SCRIPT}
echo '}' >> ${STARTUP_SCRIPT}
echo '' >> ${STARTUP_SCRIPT}
echo 'case "$1" in' >> ${STARTUP_SCRIPT}
echo '  start)' >> ${STARTUP_SCRIPT}
echo '        start' >> ${STARTUP_SCRIPT}
echo '        ;;' >> ${STARTUP_SCRIPT}
echo '' >> ${STARTUP_SCRIPT}
echo '  stop)' >> ${STARTUP_SCRIPT}
echo '        stop' >> ${STARTUP_SCRIPT}
echo '        ;;' >> ${STARTUP_SCRIPT}
echo '' >> ${STARTUP_SCRIPT}
echo '  restart)' >> ${STARTUP_SCRIPT}
echo '	stop' >> ${STARTUP_SCRIPT}
echo '        start' >> ${STARTUP_SCRIPT}
echo '        ;;' >> ${STARTUP_SCRIPT}
echo '' >> ${STARTUP_SCRIPT}
echo '  *)' >> ${STARTUP_SCRIPT}
echo '        echo $"Usage: $0 {start|stop|restart}"' >> ${STARTUP_SCRIPT}
echo '        exit 1' >> ${STARTUP_SCRIPT}
echo 'esac' >> ${STARTUP_SCRIPT}
echo '' >> ${STARTUP_SCRIPT}
echo 'exit 0' >> ${STARTUP_SCRIPT}
chmod 755 ${STARTUP_SCRIPT}

echo "Update services loading at boot time..."
update-rc.d freepbx defaults

clear
echo "##################################################################"
echo "# PART 2 - FreePBX installation : Done                           #"
echo "##################################################################"

rm /bin/sh
ln -s /bin/bash /bin/sh
sed -i "s/\/bin\/sh/\/bin\/bash/" /usr/sbin/safe_asterisk

/etc/init.d/freepbx start

beep -f 500 -n -f 1000 -n -f 1500
beep -f 500 -n -f 1000 -n -f 1500
beep -f 500 -n -f 1000 -n -f 1500
beep -f 500 -n -f 1000 -n -f 1500

##################################################################################################

###################################################################################################

###################################################################################################
clear
echo "+----------------------------------------------------------------+"
echo "| Asterisk with FreePBX installation is finished.                  |"
echo "| For running asterisk+freepbx you must use this command :       |"
echo "| # /etc/init.d/freepbx start                                    |"
echo "|                                                                |"
echo "| # you can also use amportal start stop restart etc             |"
echo "| !! NEVER LOAD ASTERISK DIRECTLY !!                             |"
echo "+----------------------------------------------------------------+"

echo "+----------------------------------------------------------------+"
echo "| Asterisk has been started goto http://${LOCAL_IP}                                    |"
echo "| Done, enjoy                                                    |"
echo "+----------------------------------------------------------------+"
